/*
 * Copyright 2002 Eric Hodel. All rights reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 * 1. Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in
 *    the documentation and/or other materials provided with the
 *    distribution.
 * 3. All advertising materials mentioning features or use of this
 *    software must display the following acknowledgement:
 *    This product includes software developed by Eric Hodel and
 *    his contributors.
 * 4. Neither the name of Eric Hodel nor the names of his
 *    contributors may be used to endorse or promote products derived
 *    from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY ERIC HODEL AND CONTRIBUTORS ``AS IS''
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED
 * TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
 * PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL ERIC HODEL OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
 * USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
 * ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
 * OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
 * OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

/* jslib */
/*
include('chrome://jslib/content/io/io.js');
include('chrome://jslib/content/rdf/rdf.js');
include('chrome://jslib/content/rdf/rdfBase.js');
include('chrome://jslib/content/rdf/rdfContainer.js');
include('chrome://jslib/content/rdf/rdfFile.js');
include('chrome://jslib/content/rdf/rdfResource.js');
*/

const CLIENTVERSION = "Mozilla-Livelizard/1.5";

/* pref strings */
const EVANGELIZE_PREF = "livelizard.evangelize";
const GOOGLEAPI_KEY_PREF = "livelizard.googleKey";
const LJ_URL_PREF = "livelizard.ljURL";
const LJ_CONN_TYPE_PREF = "livelizard.ljConnectionType";
const TZ_OFFSET_PREF = "livelizard.tzOffset";

var gLJ;
var gLJConnType;
var gGoogleAPI;
var gEditor;

/* cached data */
var moods = [];

function Init() {
  /*
   * Need to set defaults before mucking with prefs.
   * This should be moved into a per-profile RDF file
   */
  getPref().SetDefaultCharPref(LJ_URL_PREF,
    "http://www.livejournal.com/interface/flat");
  getPref().SetDefaultCharPref(LJ_CONN_TYPE_PREF, "POST");
  getPref().SetDefaultCharPref(GOOGLEAPI_KEY_PREF, "");
  getPref().SetDefaultIntPref(TZ_OFFSET_PREF, 0);
  getPref().SetDefaultBoolPref(EVANGELIZE_PREF, false);

  var googleKey = getPref().CopyCharPref(GOOGLEAPI_KEY_PREF);

  gEditor = new Editor("content-frame", "content-source", "editor-deck");

  if (googleKey != "") {
    /*document.getElementById("spellcheck").disabled = false;*/
    gGoogleAPI = new GoogleAPI(googleKey);
  }

  gLJConnType = getPref().CopyCharPref(LJ_CONN_TYPE_PREF);

  setStatusText(getString("NotLoggedIn"));
  updateDate();
  enableDisableAutoDate();
  if (loadCredentials())
    login();
  //loadMoods();
}

function Destroy() {
  saveCredentials(document.getElementById("username").value,
    document.getElementById("password").value);
}

function login() {
  var username = document.getElementById("username").value;
  var password = document.getElementById("password").value;
  var hpassword = hexMD5(password);

  if (gLJConnType == "POST") {
    var ljURL = getPref().CopyCharPref(LJ_URL_PREF);
    gLJ = new LJHTTPPostConnection(ljURL, username, null, hpassword);
    var loginResult = gLJ.login(moods.length, CLIENTVERSION);

    setStatusText(getString("LoginResponseRecieved"));

    if (loginResult) {
      var i;

      /* usejournal */
      setStatusText(getString("LoadingSharedJournals"));
      var usejournal_menulist = document.getElementById("usejournal"); 
      usejournal_menulist.removeChild(usejournal_menulist.firstChild);
      usejournal_menulist.appendChild(document.createElement("menupopup"));
      usejournal_menulist.appendItem(document.getElementById("username").value);
      for (i = 1; i < loginResult.usejournal.length; i++) {
        usejournal_menulist.appendItem(loginResult.usejournal[i]);
      }
      usejournal_menulist.selectedIndex = 0;

      /* picture keywords */
      setStatusText(getString("LoadingPictures"));
      var pickw;
      var pickw_menulist = document.getElementById("pickw");
      pickw_menulist.removeChild(pickw_menulist.firstChild);
      pickw_menulist.appendChild(document.createElement("menupopup"));
      pickw_menulist.appendItem(getString("PictureDefault"));
      for (i = 1; i < loginResult.pickw.length; i++) {
        pickw = loginResult.pickw[i];
        pickw_menulist.appendItem(pickw.keyword, pickw.keyword);
      }

      /* moods */
      setStatusText(getString("LoadingMoods"));
      var mood;
      var mood_menulist = document.getElementById("mood");
      mood_menulist.removeChild(mood_menulist.firstChild);
      mood_menulist.appendChild(document.createElement("menupopup"));
      for (i = 1; i < loginResult.moods.length; i++) {
        mood = loginResult.moods[i];
        mood_menulist.appendItem(mood.name, mood.id);
      }

      setStatusText(getString("LoginSuccessful").replace(/%username%/, ""));

      saveCredentials(username, password);
      //saveMoods();
    } else {
      setStatusText(getString("LoginFailed").replace(/%reason%/, ""));
    }
  /*
  if (message) {
    alert(message);
  }
    setStatusText(getString("LoginFailed").replace(/%reason%/,
      getString("UnableToConnect")));
  */
  }
}

function postevent() {
  var event = gEditor.content;

  event = event.replace(/&lt;lj([^&]*)&gt;/gi, '<lj$1>');
  event = event.replace(/&lt;\/lj([^&]*)&gt;/gi, '</lj$1>');

  if (getPref().GetBoolPref(EVANGELIZE_PREF))
    event += '<p style="font-size: smaller">Posted with <a href="http://livelizard.mozdev.org/">Livelizard</a></p>'

  var subject = document.getElementById("subject").value;
  var usejournal = document.getElementById("usejournal").label; 

  var pickw = document.getElementById("pickw").value;
  var mood = document.getElementById("mood").value;
  var music = document.getElementById("music").value;

  var allowcomments = document.getElementById("allowcomments").checked ? 0 : 1;
  var backdate = document.getElementById("backdate").checked ? 1 : 0;

  var date, time;
  if (document.getElementById("autodate").checked) {
    var tzOffset = getPref().GetIntPref(TZ_OFFSET_PREF);
    var now = new Date((new Date).valueOf() + 1000 * tzOffset * 3600);
    
    date = (now.getYear() + 1900) + "/" +
      (now.getMonth() + 1) + "/" + now.getDate();
    time = now.getHours() + ":" + now.getMinutes();
  } else {
    date = document.getElementById("date").value;
    time = document.getElementById("time").value;
  }
  
  var postResult = gLJ.postevent(subject, event, date, time, usejournal,
    mood, music, pickw, 1 /* autoformat off */, allowcomments, backdate);

  if (postResult) {
    setStatusText(getString("UpdateSuccessful"));
    updateDate();
    gEditor.clear();
    document.getElementById("subject").value = "";
  } else {
    setStatusText(getString("UpdateFailed").replace(/%reason%/, ""));
  }
}

/* utility functions */

function updateDate() {
  var tzOffset = getPref().GetIntPref(TZ_OFFSET_PREF);
  var now = new Date((new Date).valueOf() + 1000 * tzOffset * 3600);
  document.getElementById("date").value = (now.getYear() + 1900) +
    "/" + (now.getMonth() + 1) + "/" + now.getDate();
  document.getElementById("time").value = now.toTimeString();
}

function enableDisableAutoDate() {
  var autodate = document.getElementById("autodate");
  var date = document.getElementById("date");
  var time = document.getElementById("time");

  if (autodate.checked) {
    date.disabled = true;
    time.disabled = true;
  } else {
    date.disabled = false;
    time.disabled = false;
  }
}

/*
function loadMoods() {
}

function saveMoods() {
  var du = new DirUtils();
  var d = new Dir(du.getMozUserHomeDir());
  var file = d.append("livelizard.rdf");
  var livelizardRDF = new RDFFile(file);

  var seq = livelizardRDF.addRootSeq("urn:livelizard:moods");
  if (seq != null) {
    for (var i = 1; i <= moods.length; i++) {
      var mood = moods[i];
      var mood_id = mood[MOOD_ID];
      var mood_name = mood[MOOD_NAME];
      //alert(i + " id: " + mood_id + ", name: " + mood_name);
      var node = seq.addNode(mood_id);
      seq.setAttribute("urn:livelizard:moods:" + mood_id,
        "Name", mood_name);
    }
  }

  livelizardRDF.flush();
}
*/

function loadCredentials() {
  var loginData = {
    ljURL : getPref().CopyCharPref(LJ_URL_PREF),
    username : '',
    password : ''
  };
 
  loginData = getPassword(loginData);
  if (!loginData)
    return false;

  document.getElementById('username').value = loginData.username;
  document.getElementById('password').value = loginData.password;

  return true;
}

function saveCredentials(username, password) {
  var loginData = {
    ljURL : getPref().CopyCharPref(LJ_URL_PREF),
    username : username,
    password : password,
    savePassword : document.getElementById('savePassword').checked
  };

  return savePassword(loginData);
}

function showPrefsDialog() {
  goPreferences("livelizard",
    "chrome://livelizard/content/pref/prefLivelizard.xul",
    "livelizard.xul");
}

function showAboutDialog() {
  window.openDialog("chrome://livelizard/content/about.xul",
    "About Livelizard",
    "chrome,resizable=yes,height=400,width=600");
}
